/*
 *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * rre.c - handle RRE encoding.
 *
 * This file shouldn't be compiled directly.  It is included multiple times by
 * rfbproto.c, each time with a different definition of the macro BPP.  For
 * each value of BPP, this file defines a function which handles an RRE
 * encoded rectangle with BPP bits per pixel.
 */

#define HandleRREBPP CONCAT2E(HandleRRE,BPP)
#define CARDBPP CONCAT2E(CARD,BPP)

static Bool
HandleRREBPP (int rx, int ry, int rw, int rh)
{
  rfbRREHeader hdr;
  int i;
  CARDBPP pix;
  rfbRectangle subrect;
#if 1
  GdkGCValues gdkgcv;
#endif

  if (!ReadFromMultiVNCServer((char *)&hdr, sz_rfbRREHeader))
    return False;

  hdr.nSubrects = Swap32IfLE(hdr.nSubrects);

  if (!ReadFromMultiVNCServer((char *)&pix, sizeof(pix)))
    return False;

#if (BPP == 8)
  gdkgcv.foreground.pixel = (appData.useBGR233 ? BGR233ToPixel[pix] : pix);  
#else
  gdkgcv.foreground.pixel = pix;
#endif

  gdk_gc_set_foreground(gdkGC, &gdkgcv.foreground);
  draw_rectangle_for_client(gdkpixmap[0][0], gdkGC, TRUE, rx, ry, rw, rh, gdkgcv.foreground.pixel);

  for (i = 0; i < hdr.nSubrects; i++) {
    if (!ReadFromMultiVNCServer((char *)&pix, sizeof(pix)))
      return False;

    if (!ReadFromMultiVNCServer((char *)&subrect, sz_rfbRectangle))
      return False;

    subrect.x = Swap16IfLE(subrect.x);
    subrect.y = Swap16IfLE(subrect.y);
    subrect.w = Swap16IfLE(subrect.w);
    subrect.h = Swap16IfLE(subrect.h);

#if (BPP == 8)
    gdkgcv.foreground.pixel = (appData.useBGR233 ? BGR233ToPixel[pix] : pix);
#else
    gdkgcv.foreground.pixel = pix;
#endif

    gdk_gc_set_foreground(gdkGC, &gdkgcv.foreground);
    draw_rectangle_for_client(gdkpixmap[0][0], gdkGC, TRUE, rx + subrect.x,
		       ry + subrect.y, subrect.w, subrect.h, gdkgcv.foreground.pixel);
  }

  return True;
}
